﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Weather
{
    public class WeatherSystem
    {
        public GraphicsDevice dev { get; set; }
        public ParticleSystem system { get; set; }


        public Vector2 pos { get; set; }
        public Rectangle rectanglePos { get; set; }
        public Texture2D tex { get; set; }
        public int whereX { get; set; }
        public int whereY { get; set; }
        public float rotation { get; set; }
        public float Yspeed { get; set; }
        public float Xspeed { get; set; }
        public weather weatherState { get; set; }
        private weather w;
        public enum weather
        {
            snowing,
            raining,
            storming,
            none
        }

        public Snow snow;
        public Wind wind;
        public Rain rain;

        public WeatherSystem(weather weather, GraphicsDevice device, int screenWidth, int screenHeight)
        {
            this.weatherState = weather;
            this.dev = device;
            this.whereX = screenWidth;
            this.whereY = screenHeight;
            this.w = weather.none;

            this.system = new ParticleSystem(this.dev);
        }

        private void setNull()
        {
            this.pos = Vector2.Zero;
            this.rectanglePos = Rectangle.Empty;
            this.tex = null;
        }

        public void Update(int random, weather weatherstate)
        {
            weatherState = weatherstate;
            switch (weatherState)
            {
                case (weather.snowing):
                    if (w == weatherState)
                    {
                        updateSnow();
                    }
                    else
                    {
                        initSnow(random);
                    }
                    break;

                case (weather.storming):
                    if (w == weatherState)
                    {
                        updateWind();
                    }
                    else
                    {
                        initWind(random);
                    }
                    break;
                case (weather.raining):
                    if (w == weatherState)
                    {
                        updateRain();
                    }
                    else
                    {
                        initRain(random);
                    }
                    break;

                default:
                    break;
            }
            w = weatherState;
        }


        #region snow
        private void updateSnow()
        {
            snow.Update();
            this.rectanglePos = snow.rectanglePos;
            this.pos = snow.pos;
            this.tex = snow.tex;
            this.rotation = snow.rotationSpeed;
        }

        private void initSnow(int r, float rotation = 0.01f, float yspeed = 2f, float xspeed = 0.2f)
        {
            setNull();
            Random random = new Random(((r * r * r) / 2) - r);

            this.tex = system.CreateEmptySprite(5, 5, Color.White);
            this.pos = new Vector2(random.Next(0, whereX), random.Next(0, whereY));
            this.rotation = rotation;
            this.Xspeed = xspeed;
            this.Yspeed = yspeed;

            snow = new Snow(this.pos, this.tex, this.rotation, whereX, whereY, this.Yspeed, this.Xspeed);
            updateSnow();
        }
        #endregion

        #region wind
        private void updateWind()
        {
            wind.Update();
            this.rectanglePos = wind.rectanglePos;
            this.pos = wind.pos;
            this.tex = wind.tex;
            this.rotation = wind.rotationSpeed;
        }

        private void initWind(int r, float rotation = 0.002f, float yspeed = 0.01f, float xspeed = 5f)
        {
            setNull();
            Random random = new Random(((r * r * r) / 2) - r);

            this.tex = system.CreateEmptySprite(2, 2, Color.GhostWhite);
            this.pos = new Vector2(random.Next(0, whereX), random.Next(0, whereY));
            this.rotation = rotation;
            this.Xspeed = xspeed;
            this.Yspeed = yspeed;

            wind = new Wind(this.pos, this.tex, this.rotation, whereX, whereY, this.Yspeed, this.Xspeed);
            updateWind();
        }
        #endregion

        #region rain

        private void updateRain()
        {
            rain.Update();
            this.rectanglePos = rain.rectanglePos;
            this.pos = rain.pos;
            this.tex = rain.tex;
            this.rotation = rain.rotationSpeed;
        }

        private void initRain(int r, float rotation = 0.1f, float yspeed = 4f, float xspeed = 0.8f)
        {
            setNull();
            Random random = new Random(((r * r * r) / 2) - r);

            this.tex = system.CreateEmptySprite(2, 4, Color.Blue);
            this.pos = new Vector2(random.Next(0, whereX), random.Next(0, whereY));
            this.rotation = rotation;
            this.Xspeed = xspeed;
            this.Yspeed = yspeed;

            rain = new Rain(this.pos, this.tex, this.rotation, this.whereX, this.whereY, this.Yspeed, this.Xspeed);
            updateRain();
        }
        #endregion
    }
}
