using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Weather
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class mainClass : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        ParticleSystem particle;


        //player
        public Texture2D playerTex;
        public Vector2 playerPos;


        //weather
        public int amountOfParticles = 100;
        public float scale = 1f;
        WeatherSystem[] weather;
        WeatherSystem.weather weatherState = WeatherSystem.weather.raining;


        //div
        int screenWidth = 800;
        int screenHeight = 600;
        KeyboardState oldState;
        string path = string.Format(@"{0}\OPTIONAL.INI", System.Reflection.Assembly.GetExecutingAssembly().Location.Replace("Weather.exe", ""));


        public mainClass()
        {
            graphics = new GraphicsDeviceManager(this);
        }



        void getCFG()
        {
            if (System.IO.File.Exists(path))
            {
                IniFile ini = new IniFile(path);
                amountOfParticles = Convert.ToInt32(ini.ReadValue("GAME", "WEATHER_PARTICLES"));
                scale = (float)Convert.ToInt32(ini.ReadValue("GAME", "WEATHER_PARTICLES_SCALE"));
            }
        }



        bool getFullscreen()
        {
            if (System.IO.File.Exists(path))
            {
                IniFile ini = new IniFile(path);
                return Convert.ToBoolean(ini.ReadValue("GAME", "FULLSCREEN"));
            }
            else
            {
                return false;
            }
        }



        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = screenWidth;
            graphics.PreferredBackBufferHeight = screenHeight;
            graphics.IsFullScreen = getFullscreen(); //false - true
            graphics.ApplyChanges();

            //set the graphicsdevice and the particleEngine and the Camera
            particle = new ParticleSystem(this.GraphicsDevice);


            //posities
            playerPos = new Vector2(screenWidth / 2, screenHeight - 40);


            //set the weathersystem
            getCFG();
            weather = new WeatherSystem[amountOfParticles];
            for (int i = 0; i < amountOfParticles; i++)
            {
                weather[i] = new WeatherSystem(weatherState, this.GraphicsDevice, screenWidth, screenHeight);
            }


            //div
            oldState = Keyboard.GetState();

            base.Initialize();
        }



        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            playerTex = particle.CreateEmptySprite(10, 40, Color.LightGray);
        }



        protected override void Update(GameTime gameTime)
        {
            //updateVoids
            updateInput();


            //weathersystem update
            for (int i = 0; i < amountOfParticles; i++)
            {
                weather[i].Update((i * i) + 2, weatherState);
            }


            base.Update(gameTime);
        }



        void updateInput()
        {
            KeyboardState newState = Keyboard.GetState();

            //afsluiten (ESCAPE)
            if (newState.IsKeyDown(Keys.Escape) && !oldState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            
            //nieuw weer (SPATIEBALK)
            if (newState.IsKeyDown(Keys.Space) && !oldState.IsKeyDown(Keys.Space))
            {
                //new weather!!!
                switch (weatherState)
                {
                    case (WeatherSystem.weather.raining):
                        weatherState = WeatherSystem.weather.snowing;
                        break;
                    case (WeatherSystem.weather.snowing):
                        weatherState = WeatherSystem.weather.storming;
                        break;
                    case (WeatherSystem.weather.storming):
                        weatherState = WeatherSystem.weather.raining;
                        break;
                }
            }

            //links (pijltje links)
            if (newState.IsKeyDown(Keys.Left) && !oldState.IsKeyDown(Keys.Left) && playerPos.X > 0)
            {
                playerPos.X -= 3;
            }

            //rechts (pijltje rechts)
            if (newState.IsKeyDown(Keys.Right) && !oldState.IsKeyDown(Keys.Right) && playerPos.X < screenWidth)
            {
                playerPos.X += 3;
            }

            //ontwikkelingshulpprogrammas (F12)
            if (newState.IsKeyDown(Keys.F12) && !oldState.IsKeyDown(Keys.F12) && !System.IO.File.Exists(path))
            {
                using (System.IO.File.Create(path)) { }
                IniFile ini = new IniFile(path);

                ini.WriteValue("GAME", "WEATHER_PARTICLES", amountOfParticles.ToString());
                ini.WriteValue("GAME", "WEATHER_PARTICLES_SCALE", scale.ToString());
                ini.WriteValue("GAME", "FULLSCREEN", "FALSE");
            }
        }


        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin();
            spriteBatch.Draw(playerTex, playerPos, Color.White);


            for (int i = 0; i < amountOfParticles; i++)
            {
                spriteBatch.Draw(weather[i].tex, weather[i].pos, null, Color.White, weather[i].rotation, Vector2.Zero, scale, SpriteEffects.None, 0);
            }


            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
